@@ -29,8 +29,9 @@ class Internal_RS485_slave_class:
         self.register_update_thread=None
         self.set_data_dict = {}
         self.base_data_dict = {}
-        self.load_data_dict()
         self.register_dict={}
+        self.load_data_dict()
+        
         self.byteorder=byteorder
         self.wordorder=wordorder
 
@@ -59,6 +60,8 @@ class Internal_RS485_slave_class:
             hr=self.holding_registers,
             ir=self.input_registers
         )
+        print(f"Slave {self.name} initialized with address {self.slave_address} and device name {self.device_name}")
+        print({"context_obj":self.slave_context})
         try:
             self.add_to_server()
         except:
@@ -136,11 +139,16 @@ class Internal_RS485_slave_class:
         elif(data_type == "UInt64"):
             pack_format = byteorder+'Q'
             unpack_format = byteorder+'Q'
+            
+        if(data_type in ["Int16", "Int32", "Int64", "UInt16", "UInt32", "UInt64"]):
+            value = int(value)  # Ensure value is an integer for these types
         
- 
+        print("Data to be converted: ", value)
         # Pack float into 32-bit binary, then unpack as hex
         hex_val= hex(struct.unpack(unpack_format, struct.pack(pack_format, value))[0])    
         
+        print("Hex value: ", hex_val)
+
         while(len(hex_val)<(2+size*4)):
             hex_val=hex_val[:2]+"0"+hex_val[2:]
         
@@ -148,11 +156,15 @@ class Internal_RS485_slave_class:
         for i in range(0, size):
             unordered_word_list.append(int(hex_val[i*4+2:i*4+4+2],16))
         
+        print("Unordered word list: ", unordered_word_list)
+
         if(wordorder == '>'):
             ordered_word_list=unordered_word_list
         elif(wordorder == '<'):
             ordered_word_list=unordered_word_list[::-1]
 
+        print("Ordered word list: ", ordered_word_list)
+
         return ordered_word_list
   
     def load_data_dict(self):
@@ -185,28 +197,42 @@ class Internal_RS485_slave_class:
         """Set data in the holding registers."""
         start_address=register[0]
         size=register[1]
+        default_byteorder=register[2]
+        default_wordorder=register[3]
+        data_type=register[4]  
+        try:
+            factor=float(register[6])
+        except:
+            factor=1.0
 
+        print(f"Setting data in holding registers at address {start_address} with size {size} and data {data}")
         if(self.byteorder not in ['Little_Edian', 'Big_Edian']):
-            byteorder=register[2]
+            byteorder=default_byteorder
         elif(self.byteorder == 'Little_Edian'):
             byteorder='<'
         elif(self.byteorder == 'Big_Edian'):
             byteorder='>'
 
         if(self.wordorder not in ['Little_Edian', 'Big_Edian']):
-            wordorder=register[2]
+            wordorder=default_wordorder
         elif(self.byteorder == 'Little_Edian'):
             wordorder='<'
         elif(self.byteorder == 'Big_Edian'):
             wordorder='>'
         
-        data_type=register[4]        
-        factor=float(register[5])
-
-        words=self.data_to_words(data*factor, byteorder=byteorder, 
-                                    wordorder=wordorder, size=size,data_type=data_type)
+        
+        try:
+            words=self.data_to_words(data*factor, byteorder=byteorder, 
+                                        wordorder=wordorder, size=size,data_type=data_type)
+        except Exception as e:
+            print(f"Error converting data to words: {e}")
+            pass
+        # print(f"Words to be set: {words}")
         for i in range(size):
             self.holding_registers.setValues(start_address+i, words[i])
+        
+        print("Register values; "+str(self.holding_registers.getValues(start_address, size)))
+        
 
     ### function to update local data dictionary in global data interface
     def update_data_in_data_interface(self):
@@ -220,124 +246,126 @@ class Internal_RS485_slave_class:
 
     def update_set_data_to_registers(self):
         """Update the Modbus registers with the set data dictionary."""
-        while(1):
-            try:
-                if(self.device_name == "WF_chiller"):
-                    reg4_data=int(self.set_data_dict["chiller_liquid_temp"]*10)
-                    reg5_data=int(self.set_data_dict["chiller_aft_temp"]*10)      
-                    reg10_bin_data= [self.set_data_dict["chiller_room_temp_probe_fail"],
-                                    self.set_data_dict["chiller_aft_probe_fail"],
-                                    self.set_data_dict["chiller_room_temp_probe_ht_fault"],
-                                    self.set_data_dict["chiller_room_temp_probe_lt_fault"]]  
-                    
-                    reg11_bin_data=[self.set_data_dict["chiller_pump_overload_fault"],
-                                    self.set_data_dict["chiller_spp_fault"],
-                                    self.set_data_dict["chiller_compressor_overload_fault"],
-                                    self.set_data_dict["chiller_hp_fault"],
-                                    self.set_data_dict["chiller_lp_fault"]]  
-                    
-                    reg12_bin_data=[self.set_data_dict["chiller_low_liquid_level_fault"],
-                                    self.set_data_dict["chiller_liquid_line_high_temp_fault"],
-                                    self.set_data_dict["chiller_high_temp_fault"],
-                                    self.set_data_dict["chiller_aft_fault"]]
-                    
-                    reg13_bin_data=[self.set_data_dict["chiller_compressor_on"],
-                                    self.set_data_dict["chiller_pump_on"],
-                                    self.set_data_dict["chiller_sv_on"],
-                                    self.set_data_dict["chiller_alarm_on"]]
-                    
-
-                    reg10_data=self.convert_bin_list_to_int(reg10_bin_data)
-                    
-                    reg11_data=self.convert_bin_list_to_int(reg11_bin_data)
-                    
-                    reg12_data=self.convert_bin_list_to_int(reg12_bin_data)
-                    
-                    reg13_data=self.convert_bin_list_to_int(reg13_bin_data)
-                    
-                    reg28_data=int(self.set_data_dict["chiller_set_high_set"]*10) 
-                    reg29_data=int(self.set_data_dict["chiller_set_low_set"]*10) 
-                    reg30_data=int(self.set_data_dict["chiller_set_set_point"]*10) 
-                    reg34_data=int(self.set_data_dict["chiller_set_differential"]*10) 
-                    reg35_data=int(self.set_data_dict["chiller_set_high_temp_alarm"]*10) 
-                    reg36_data=int(self.set_data_dict["chiller_set_low_temp_alarm"]*10) 
-                    reg38_data=int(self.set_data_dict["chiller_set_aft_set_temp"]*10) 
-                    reg39_data=int(self.set_data_dict["chiller_set_aft_differential"]*10) 
-                    reg51_data=int(self.set_data_dict["chiller_remote_start"]) 
-
-                    self.holding_registers.setValues(4, reg4_data)
-                    self.holding_registers.setValues(5, reg5_data)            
-                    self.holding_registers.setValues(10, reg10_data)
-                    self.holding_registers.setValues(11, reg11_data)
-                    self.holding_registers.setValues(12, reg12_data)
-                    self.holding_registers.setValues(13, reg13_data)
-                    self.holding_registers.setValues(28, reg28_data)
-                    self.holding_registers.setValues(28, reg28_data)
-                    self.holding_registers.setValues(29, reg29_data)
-                    self.holding_registers.setValues(30, reg30_data)
-                    self.holding_registers.setValues(34, reg34_data)
-                    self.holding_registers.setValues(35, reg35_data)
-                    self.holding_registers.setValues(36, reg36_data)
-                    self.holding_registers.setValues(38, reg38_data)
-                    self.holding_registers.setValues(39, reg39_data)
-                    self.holding_registers.setValues(51, reg51_data)
+        
+        try:
+            if(self.device_name == "WF_chiller"):
+                reg4_data=int(self.set_data_dict["chiller_liquid_temp"]*10)
+                reg5_data=int(self.set_data_dict["chiller_aft_temp"]*10)      
+                reg10_bin_data= [self.set_data_dict["chiller_room_temp_probe_fail"],
+                                self.set_data_dict["chiller_aft_probe_fail"],
+                                self.set_data_dict["chiller_room_temp_probe_ht_fault"],
+                                self.set_data_dict["chiller_room_temp_probe_lt_fault"]]  
+                
+                reg11_bin_data=[self.set_data_dict["chiller_pump_overload_fault"],
+                                self.set_data_dict["chiller_spp_fault"],
+                                self.set_data_dict["chiller_compressor_overload_fault"],
+                                self.set_data_dict["chiller_hp_fault"],
+                                self.set_data_dict["chiller_lp_fault"]]  
+                
+                reg12_bin_data=[self.set_data_dict["chiller_low_liquid_level_fault"],
+                                self.set_data_dict["chiller_liquid_line_high_temp_fault"],
+                                self.set_data_dict["chiller_high_temp_fault"],
+                                self.set_data_dict["chiller_aft_fault"]]
+                
+                reg13_bin_data=[self.set_data_dict["chiller_compressor_on"],
+                                self.set_data_dict["chiller_pump_on"],
+                                self.set_data_dict["chiller_sv_on"],
+                                self.set_data_dict["chiller_alarm_on"]]
                 
-                elif(self.device_name == "WF_heater"):
 
-                    relay_encoded_list=["heater_relay_sts_alarm","heater_relay_sts_compressor"]
-                    fault_encoded_list=["heater_fault_sts_probe_fail_low","heater_fault_sts_probe_fail_high",
-                                            "heater_fault_sts_ht","heater_fault_sts_lt","heater_fault_water_level_low",
-                                            "heater_fault"]
-                    
-                    reg11_data=self.convert_bin_list_to_int([self.set_data_dict["heater_relay_sts_alarm"],
-                                                        self.set_data_dict["heater_relay_sts_compressor"]])
-                                                
+                reg10_data=self.convert_bin_list_to_int(reg10_bin_data)
+                
+                reg11_data=self.convert_bin_list_to_int(reg11_bin_data)
+                
+                reg12_data=self.convert_bin_list_to_int(reg12_bin_data)
+                
+                reg13_data=self.convert_bin_list_to_int(reg13_bin_data)
+                
+                reg28_data=int(self.set_data_dict["chiller_set_high_set"]*10) 
+                reg29_data=int(self.set_data_dict["chiller_set_low_set"]*10) 
+                reg30_data=int(self.set_data_dict["chiller_set_set_point"]*10) 
+                reg34_data=int(self.set_data_dict["chiller_set_differential"]*10) 
+                reg35_data=int(self.set_data_dict["chiller_set_high_temp_alarm"]*10) 
+                reg36_data=int(self.set_data_dict["chiller_set_low_temp_alarm"]*10) 
+                reg38_data=int(self.set_data_dict["chiller_set_aft_set_temp"]*10) 
+                reg39_data=int(self.set_data_dict["chiller_set_aft_differential"]*10) 
+                reg51_data=int(self.set_data_dict["chiller_remote_start"]) 
+
+                self.holding_registers.setValues(4, reg4_data)
+                self.holding_registers.setValues(5, reg5_data)            
+                self.holding_registers.setValues(10, reg10_data)
+                self.holding_registers.setValues(11, reg11_data)
+                self.holding_registers.setValues(12, reg12_data)
+                self.holding_registers.setValues(13, reg13_data)
+                self.holding_registers.setValues(28, reg28_data)
+                self.holding_registers.setValues(28, reg28_data)
+                self.holding_registers.setValues(29, reg29_data)
+                self.holding_registers.setValues(30, reg30_data)
+                self.holding_registers.setValues(34, reg34_data)
+                self.holding_registers.setValues(35, reg35_data)
+                self.holding_registers.setValues(36, reg36_data)
+                self.holding_registers.setValues(38, reg38_data)
+                self.holding_registers.setValues(39, reg39_data)
+                self.holding_registers.setValues(51, reg51_data)
             
-                    reg12_data=self.convert_bin_list_to_int([self.set_data_dict["heater_fault_sts_probe_fail_low"],
-                                                        self.set_data_dict["heater_fault_sts_probe_fail_high"],
-                                                        self.set_data_dict["heater_fault_sts_ht"],
-                                                        self.set_data_dict["heater_fault_sts_lt"],
-                                                        self.set_data_dict["heater_fault_water_level_low"],
-                                                        self.set_data_dict["heater_fault"]])
-                    
-                    self.holding_registers.setValues(self.set_data_dict['relay_status'], reg11_data)
-                    self.holding_registers.setValues(self.set_data_dict['fault_status'], reg12_data)
+            elif(self.device_name == "WF_heater"):
 
-                    for k in self.set_data_dict.keys():
-                        if(k not in relay_encoded_list and k not in fault_encoded_list):
-                            reg_data=int(self.set_data_dict[k]*10)
-                            self.holding_registers.setValues(self.register_dict[k], reg_data)
-
-                elif(self.device_name == "LT_AC_EM"):
-                    for k in self.set_data_dict.keys():  
-                        if(k=="em_k_watt_hour"):
-                            in_data=self.set_data_dict[k]*1000
-                        else:
-                            in_data=self.set_data_dict[k]
-                        self.set_data_in_holding_registers(in_data,self.register_dict[k])
-
-                elif(self.device_name == "A9MEM3250_AC_EM"):
-                    for k in self.set_data_dict.keys():  
-                        in_data=self.set_data_dict[k]
-                        self.set_data_in_holding_registers(in_data,self.register_dict[k])
+                relay_encoded_list=["heater_relay_sts_alarm","heater_relay_sts_compressor"]
+                fault_encoded_list=["heater_fault_sts_probe_fail_low","heater_fault_sts_probe_fail_high",
+                                        "heater_fault_sts_ht","heater_fault_sts_lt","heater_fault_water_level_low",
+                                        "heater_fault"]
                 
-                elif(self.device_name == "PD195Z_CD31F_DC_EM"):
-                    for k in self.set_data_dict.keys():  
+                reg11_data=self.convert_bin_list_to_int([self.set_data_dict["heater_relay_sts_alarm"],
+                                                    self.set_data_dict["heater_relay_sts_compressor"]])
+                                            
+        
+                reg12_data=self.convert_bin_list_to_int([self.set_data_dict["heater_fault_sts_probe_fail_low"],
+                                                    self.set_data_dict["heater_fault_sts_probe_fail_high"],
+                                                    self.set_data_dict["heater_fault_sts_ht"],
+                                                    self.set_data_dict["heater_fault_sts_lt"],
+                                                    self.set_data_dict["heater_fault_water_level_low"],
+                                                    self.set_data_dict["heater_fault"]])
+                
+                self.holding_registers.setValues(self.set_data_dict['relay_status'], reg11_data)
+                self.holding_registers.setValues(self.set_data_dict['fault_status'], reg12_data)
+
+                for k in self.set_data_dict.keys():
+                    if(k not in relay_encoded_list and k not in fault_encoded_list):
+                        reg_data=int(self.set_data_dict[k]*10)
+                        self.holding_registers.setValues(self.register_dict[k], reg_data)
+
+            elif(self.device_name == "LT_AC_EM"):
+                for k in self.set_data_dict.keys():  
+                    if(k=="em_k_watt_hour"):
+                        in_data=self.set_data_dict[k]*1000
+                    else:
                         in_data=self.set_data_dict[k]
-                        self.set_data_in_holding_registers(in_data,self.register_dict[k])
+                    self.set_data_in_holding_registers(in_data,self.register_dict[k])
+
+            elif(self.device_name == "A9MEM3250_AC_EM"):
+                for k in list(self.set_data_dict.keys()):                      
+                    in_data=self.set_data_dict[k]
+                    print(f"Setting data for {k} in holding registers: {in_data}")
+                    self.set_data_in_holding_registers(in_data,self.register_dict[k])
+            
+            elif(self.device_name == "PD195Z_CD31F_DC_EM"):
+                for k in self.set_data_dict.keys():  
+                    in_data=self.set_data_dict[k]
+                    self.set_data_in_holding_registers(in_data,self.register_dict[k])
 
                         
-            except Exception as e:
-                # print(f"Error updating registers: {e}")
-                pass
+        except Exception as e:
+            print(f"Error updating registers: {e}")
+            pass
 
-            sleep(0.1)
+            
 
     def set_data(self, key, value):
         """Set a value in the data dictionary."""
         with self._lock:
             if key in self.set_data_dict:
                 self.set_data_dict[key] = value
+                self.set_data_in_holding_registers(value,self.register_dict[key])
             else:
                 print(f"Key {key} not found in set_data_dict.")
        
