@@ -58,8 +58,9 @@ class Internal_RS485_slave_server_class:
                                                 defer_start=False,framer=ModbusRtuFramer, port=self.hw_channal,baudrate=self.baudrate,
                                                 parity=p,stopbits=self.stopbits,timeout=self.timeout)
                 
-                print(f"Started Modbus slave server on {self.hw_channal} with baudrate {self.baudrate}, parity {p}, stopbits {self.stopbits}, timeout {self.timeout}")
-                                                
+                print(f"RS485 slave server stopped")
+            else:
+                break # to stop the server thread in case of stop_server_flag=1                                    
             
             sleep(0.1) # sleep for 100ms to avoid busy waiting
     
